'''
Created on Dec 7, 2013

@author: kostia
'''

cimport cython
cimport numpy
import numpy


@cython.boundscheck(False) 
def chain_inference(numpy.ndarray[double, ndim = 2] transition_weights, numpy.ndarray[double, ndim = 2] unary_weights):
    
    cdef int n_states = transition_weights.shape[0]
    cdef int n_examples = unary_weights.shape[1]
    
    cdef numpy.ndarray cur = numpy.zeros([n_states], dtype=numpy.double)
    cdef numpy.ndarray prev = numpy.zeros([n_states], dtype=numpy.double)
    cdef numpy.ndarray parent = -1 * numpy.ones([n_states,n_examples], dtype=numpy.int32)
    
    cdef int i,j,k  ## iterator variables
    
    #initialize first layer
    for k in xrange(n_states): cur[k] = unary_weights[k,0]
    
    #run dp
    for i in xrange(1,n_examples):
        
        for k in xrange(n_states): 
            prev[k] = cur[k]
            cur[k] = -numpy.inf
            
        for k in xrange(n_states):
            for j in xrange(n_states):
                if cur[k] < prev[j] + transition_weights[j,k] + unary_weights[k,i]:
                    cur[k] = prev[j] + transition_weights[j,k] + unary_weights[k,i]
                    parent[k,i] = j
        
    # find last maximum index
    cdef numpy.ndarray map = -1 * numpy.ones([n_examples], dtype=numpy.int32) 
    map[-1] = numpy.argmax(cur)
    for i in xrange(n_examples-1,0,-1):
        map[i-1] = parent[map[i],i]
    
    return map

@cython.boundscheck(False) 
def eval_transition_subgradient_on_chain(numpy.ndarray[int, ndim = 1] labeling, n_states):    
    
    cdef int n_observations = labeling.shape[0]
    cdef numpy.ndarray transition_subgrad = numpy.zeros([n_states, n_states], dtype=numpy.double)
    
    cdef int i
    
    for i in xrange(1,n_observations):        
        transition_subgrad[labeling[i-1],labeling[i]] += 1        
        
    return transition_subgrad


@cython.boundscheck(False) 
def eval_emission_subgradient_on_chain(numpy.ndarray[int, ndim = 1] labeling, n_states, numpy.ndarray[double, ndim = 2] X):
    
    cdef int observation_dim = X.shape[0]    
    cdef numpy.ndarray emission_subgrad = numpy.zeros([n_states,observation_dim], dtype=numpy.double)
    cdef int n_observations = labeling.shape[0]
    cdef int i,y
    
    for i in xrange(n_observations):
        y = labeling[i]
        for k in xrange(observation_dim):
            emission_subgrad[y,k] += X[k,i]
    
    return emission_subgrad