cimport cython
cimport numpy
import numpy
from libcpp.vector cimport vector
from libc.stdint cimport uint32_t, uint16_t
from scipy.sparse import csc_matrix

cdef extern from "liblbp_v2.cpp":
    uint32_t liblbp_pyr_get_dim(uint16_t img_nRows, uint16_t img_nCols, uint16_t nPyramids);
    void liblbp_pyr_features_sparse(uint32_t* vec, uint32_t vec_nDim, uint32_t* img, uint16_t img_nRows, uint16_t img_nCols);
#    vector[int] solveMultiLabelProblem(const int, double*, const int, double *, const int, int *, const int);

# @cython.boundscheck(False)
# numpy.ndarray[int, mode="c", ndim = 2] edges,
def buildLbp(numpy.ndarray[int, ndim = 2] image, long height_of_pyramid):
    cdef int num_rows = image.shape[0]
    cdef int num_cols = image.shape[1]
    cdef int size = num_rows * num_cols

    cdef uint32_t num_features = liblbp_pyr_get_dim(num_rows,num_cols,height_of_pyramid)
    cdef int nnz = num_features / 256

    cdef numpy.ndarray[uint32_t, mode="c", ndim=1] imgclone = numpy.zeros(size, dtype=numpy.uint32)
    cdef uint32_t* img_ptr = <uint32_t*>imgclone.data
    cdef int cnt = 0

    for j in range(num_cols):
        for i in range(num_rows):
            img_ptr[cnt] = image[i, j]
            cnt += 1

    cdef numpy.ndarray[uint32_t, mode="c", ndim=1] p_index = numpy.zeros(nnz).astype(numpy.uint32)
    liblbp_pyr_features_sparse(<uint32_t*>p_index.data, nnz, img_ptr, num_rows, num_cols)

    cdef numpy.ndarray[double, mode="c", ndim=1] val = numpy.ones(nnz)
    cdef numpy.ndarray[int, mode="c", ndim=1] col = numpy.zeros(nnz).astype(numpy.int32)
    cdef numpy.ndarray[int, mode="c", ndim=1] row = numpy.zeros(nnz).astype(numpy.int32)

    for i in range(nnz):
        row[i] = p_index[i]

    return csc_matrix((val, (row, col)), shape=(num_features, 1))